﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using PS4000A_sigAWG;

namespace PicoScopeReader
{
    public partial class Form1 : Form
    {
        bool _ready = false;
        ushort[] inputRanges = { 10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000, 20000, 50000 };
        int _noOfSamples = 0;

        public Form1()
        {
            InitializeComponent();

            cbA.SelectedIndex = 0;
            cbB.SelectedIndex = 0;
            cbC.SelectedIndex = 0;
            cbD.SelectedIndex = 0;
            cbE.SelectedIndex = 0;
            cbF.SelectedIndex = 0;
            cbG.SelectedIndex = 0;
            cbH.SelectedIndex = 0;

            cbUnits.SelectedIndex = 5;
            tbMaxFileSizeMB.Text = "200";
            tbFrequencyHz.Text = "5000000";
            tbMeasureTime.Text = "3";

            tbPath.Enabled = true;
            tbFrequencyHz.Enabled = true;
            tbMaxFileSizeMB.Enabled = true;
            tbMeasureTime.Enabled = true;
            btnStart.Enabled = true;

            rbBin.Checked = true;
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            int status = 0;         
            short handle = 0;       
            uint totalNoOfSamples;   // number of samples for one channel
            int frequencyHz;        // sampling frequency in Hz
            decimal frequencyS;         // sampling frequency in seconds
            uint frequency;          // frequency in part of seconds ex. 10 nanoseconds
            int noOfChannels = 0;       // number of channels
            Int64 maxFileSizeMB;      // max size of result file, if we have more data, create more files
            uint mMeasureTime;    // measure time in our unit 
            double mMeasureTimeInS; // measure time in seconds
            Imports.Range[] combobox_values = new Imports.Range[8];
            PinnedArray<short>[] minPinned = new PinnedArray<short>[8];
            PinnedArray<short>[] maxPinned = new PinnedArray<short>[8];
            Imports.ps4000aBlockReady _callbackDelegate;
            uint timebase;           // n -> 80MHz / (n+1)
            Imports.ReportedTimeUnits unit; // our unit
            int timeIndisposed;
            int fileCounter = 1;

                #region channels counting
                if (cbA.SelectedIndex > 0) noOfChannels++;
                if (cbB.SelectedIndex > 0) noOfChannels++;
                if (cbC.SelectedIndex > 0) noOfChannels++;
                if (cbD.SelectedIndex > 0) noOfChannels++;
                if (cbE.SelectedIndex > 0) noOfChannels++;
                if (cbF.SelectedIndex > 0) noOfChannels++;
                if (cbG.SelectedIndex > 0) noOfChannels++;
                if (cbH.SelectedIndex > 0) noOfChannels++;
                #endregion

                #region measure parameters

                frequencyHz = Convert.ToInt32(tbFrequencyHz.Text);
                maxFileSizeMB = (long)(Convert.ToInt64(tbMaxFileSizeMB.Text) * (long)1000000);
                mMeasureTime = Convert.ToUInt32(tbMeasureTime.Text);

                switch (cbUnits.SelectedIndex)
                {
                    case 0:
                        mMeasureTimeInS = mMeasureTime * Math.Pow(10, -15);
                        break;
                    case 1:
                        mMeasureTimeInS = mMeasureTime * Math.Pow(10, -12);
                        break;
                    case 2:
                        mMeasureTimeInS = mMeasureTime * Math.Pow(10, -9);
                        break;
                    case 3:
                        mMeasureTimeInS = mMeasureTime * Math.Pow(10, -6);
                        break;
                    case 4:
                        mMeasureTimeInS = mMeasureTime * Math.Pow(10, -3);
                        break;
                    default:
                        mMeasureTimeInS = mMeasureTime;
                        break;
                }

                int n = 80000000 / frequencyHz - 1;// (80MHz / F) - 1, bo F = 80MHz / (n+1)
                timebase = Convert.ToUInt32(n);
                totalNoOfSamples = Convert.ToUInt32(frequencyHz * mMeasureTimeInS); // obliczenie rozmiaru bufora
                frequencyS = (1 / (decimal)frequencyHz);

                decimal temp = frequencyS;
                int countUnit = 0;
                while (temp % 1 != 0)
                {
                    temp *= 1000; // *10^3
                    countUnit++;
                }
                switch (countUnit)
                {
                    case 0:
                        unit = Imports.ReportedTimeUnits.Seconds;
                        break;
                    case 1:
                        unit = Imports.ReportedTimeUnits.MilliSeconds;
                        break;
                    case 2:
                        unit = Imports.ReportedTimeUnits.MicroSeconds;
                        break;
                    case 3:
                        unit = Imports.ReportedTimeUnits.NanoSeconds;
                        break;
                    case 4:
                        unit = Imports.ReportedTimeUnits.PicoSeconds;
                        break;
                    default:
                        unit = Imports.ReportedTimeUnits.FemtoSeconds;
                        break;
                }
                frequency = Convert.ToUInt32(temp);

                #endregion

                if (check(noOfChannels, totalNoOfSamples))
                {

                    while (handle == 0)
                    {
                        status = Imports.OpenUnit(out handle, null);
                    }

                    //if not usb3 will need to set up systrem to get  the opwer from usb 2.0
                    if (status != 0)
                    {
                        btnStart.Enabled = true;
                        tbPath.Enabled = true;

                        status = Imports.ps4000aChangePowerSource(handle, status);
                    }

                    #region file name
                    string path = tbPath.Text;
                    if (path.Length == 0) path = Application.StartupPath.ToString();
                    string currentDate = DateTime.Now.ToString("yyyyMMdd_HHmmss");
                    string currentDateNormal = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                    string filePathBegin = path + "\\" + currentDate + "_";
                    string filePath = "";// filePathBegin + "channel_" + Config.CHANNEL_NAMES[0] + "_" + fileCounter + ".bin";
                    string filePathCSVBegin = path + "\\" + currentDate + "_";
                    string filePathCSV = filePathCSVBegin + fileCounter + ".csv";
                    #endregion

                    #region chanel settings
                    combobox_values[0] = (Imports.Range)(cbA.SelectedIndex - 1);
                    combobox_values[1] = (Imports.Range)(cbB.SelectedIndex - 1);
                    combobox_values[2] = (Imports.Range)(cbC.SelectedIndex - 1);
                    combobox_values[3] = (Imports.Range)(cbD.SelectedIndex - 1);
                    combobox_values[4] = (Imports.Range)(cbE.SelectedIndex - 1);
                    combobox_values[5] = (Imports.Range)(cbF.SelectedIndex - 1);
                    combobox_values[6] = (Imports.Range)(cbG.SelectedIndex - 1);
                    combobox_values[7] = (Imports.Range)(cbH.SelectedIndex - 1);

                    for (int ch = 0; ch < 8; ch++)
                    {
                        status = Imports.SetChannel(handle,                                         //the device handle
                                       Imports.Channel.CHANNEL_A + ch,                              // the channel you wish to set
                                       (short)(combobox_values[ch] != (Imports.Range)(-1) ? 1 : 0), // is enabled if off is not selected
                                       (short)1,                                                    // always DC coupled
                                       combobox_values[ch] != (Imports.Range)(-1) ? combobox_values[ch] : (Imports.Range)(0),  // -1 is not a correct range will cause an error
                                        0); // no analogue offset 
                    }
                    #endregion

                    #region triggers
                    if ((status = Imports.SetSimpleTrigger(handle, 0, Imports.Channel.CHANNEL_A, 0, Imports.ThresholdDirection.Rising, 0, 0)) != 0)// disabling trigger
                    {
                        MessageBox.Show("Cannot set trigger, Error code :" + status.ToString(),
                                        "Error Setting Trigger",
                                        MessageBoxButtons.OK,
                                        MessageBoxIcon.Error);
                    }
                    #endregion

                    totalNoOfSamples = 50000000;

                    #region ustawienie buforow
                    for (int i = 0; i < 8; i++)
                    {
                        short[] minBuffers;
                        short[] maxBuffers;
                        if (combobox_values[i] > 0) // buffers for our channels
                        {
                            minBuffers = new short[totalNoOfSamples];
                            maxBuffers = new short[totalNoOfSamples];
                            minPinned[i] = new PinnedArray<short>(minBuffers);
                            maxPinned[i] = new PinnedArray<short>(maxBuffers);
                            status = Imports.SetDataBuffers(handle, (Imports.Channel)i, maxBuffers, minBuffers, (int)totalNoOfSamples, 0, Imports.DownSamplingMode.None);
                        }
                        else
                        {
                            minBuffers = new short[0];
                            maxBuffers = new short[0];
                            minPinned[i] = new PinnedArray<short>(minBuffers);
                            maxPinned[i] = new PinnedArray<short>(maxBuffers);
                            status = Imports.SetDataBuffers(handle, (Imports.Channel)i, maxBuffers, minBuffers, (int)1, 0, Imports.DownSamplingMode.None);
                        }
                    }
                    #endregion

                    _ready = false;
                    _callbackDelegate = BlockCallback;

                    status = (short)Imports.RunStreaming(handle, ref frequency, unit, 0, 0, false, 1, Imports.DownSamplingMode.None, 15000000);

                    if (status != 0)
                    {
                        MessageBox.Show("Error running block :" + status.ToString(),
                                            "Error runblock",
                                            MessageBoxButtons.OK,
                                            MessageBoxIcon.Error);
                      //  MessageBox.Show("Łączna liczba próbek przekracza rozmiar bufora! Zmiejsz liczbę kanałów, czas próbkowania lub częstotliwość próbkowania.");
                        Imports.CloseUnit(handle);
                        return;
                    }


                    Thread.Sleep(1000);

                    while (true)
                    {

                    short overflow;
                    status = Imports.GetStreamingLatestValues(handle, StreamCallback, IntPtr.Zero); 


                    if (_ready)
                    {
                     

                        if (rbBin.Checked)
                        {
                            String headerInfo = "";
                            headerInfo += "# Dane pobrane z urzadzenia PicoScope 4824" + Config.NEW_LINE;
                            headerInfo += "# Plik binarny zawiera liczby typu signed int16" + Config.NEW_LINE;
                            headerInfo += "# Data poczatku rejestracji: " + currentDateNormal + Config.NEW_LINE;
                            headerInfo += "# Liczba kanalow: " + noOfChannels + Config.NEW_LINE;
                            headerInfo += "# Czestotliwosc probkowania: " + frequencyHz + " [Hz]" + Config.NEW_LINE;
                            headerInfo += "# Czestotliwosc probkowania: " + frequency + " [" + Config.SECOND_TYPES[countUnit] + "]" + Config.NEW_LINE;
                            headerInfo += "# Czas pomiaru: " + mMeasureTimeInS + " [" + Config.SECOND_TYPES[5 - cbUnits.SelectedIndex] + "]" + Config.NEW_LINE;
                            headerInfo += "# Liczba probek na kanal: " + totalNoOfSamples + Config.NEW_LINE;
                            headerInfo += "# Laczna liczba probek: " + totalNoOfSamples * noOfChannels + Config.NEW_LINE;

                            string channelInfo = "";
                            for (int i = 0; i < 8; i++)
                            {
                                if ((combobox_values[i] != (Imports.Range)(-1))) channelInfo += Config.CHANNEL_NAMES[i] + Config.DELIMITER;
                            }
                            headerInfo += "# " + channelInfo + Config.NEW_LINE;

                            Encoding en = Encoding.ASCII;
                            int headerBytesCount = en.GetByteCount(headerInfo);
                            byte[] freeBytesForHeader = new byte[Config.BYTES_FOR_HEADER - headerBytesCount];

                            for (int ch = 0; ch < 8; ch++)
                            {
                                filePath = filePathBegin + "channel_" + Config.CHANNEL_NAMES[ch] + "_" + fileCounter + ".bin";
                                //using (FileStream fs = File.Open(filePath, FileMode.Append))
                                //{
                                //    AddText(fs, headerInfo);
                                //    fs.Write(freeBytesForHeader, 0, freeBytesForHeader.Length);
                                //}
                                for (int f = 0; f < _noOfSamples; f = f + 1000000)
                                {
                                    if (File.Exists(filePath))
                                    {
                                        FileInfo file = new FileInfo(filePath);
                                        if ((file.Length + (2000000 * noOfChannels)) > maxFileSizeMB) // czy rozmiar pliku + max rozmiar nastepnego przebiegu jest mnijszy od zakladanego rozmiaru, 2000000 bo 2 bajty ma jeden pomiar * 1000000 bo tyle skokow jest w petli for
                                        {
                                            fileCounter++;
                                            filePath = filePathBegin + "channel_" + Config.CHANNEL_NAMES[ch] + "_" + fileCounter + ".bin";
                                        }
                                    }
                                    using (FileStream fs = File.Open(filePath, FileMode.Append))
                                    {
                                        for (int i = 0; i < 1000000; i++)
                                        {
                                            if (combobox_values[ch] != (Imports.Range)(-1))
                                            {
                                                byte[] valueBytes = BitConverter.GetBytes((short)adc_to_mv(maxPinned[ch].Target[f + i], combobox_values[ch]));
                                                fs.Write(valueBytes, 0, valueBytes.Length);
                                            }
                                        }
                                    }
                                }
                                FileInfo fileChannel = new FileInfo(filePath);
                                if (fileChannel.Length == 400) File.Delete(filePath);

                            }
                        }
                        else if (rbCSV.Checked)
                        {
                            byte[] freeBytesForHeader;
                            using (StreamWriter sw = new StreamWriter(filePathCSV, true))
                            {
                                String headerInfo = "";
                                headerInfo += "# Dane pobrane z urzadzenia PicoScope 4824" + Config.NEW_LINE;
                                headerInfo += "# Data poczatku rejestracji: " + currentDateNormal + Config.NEW_LINE;
                                headerInfo += "# Liczba kanalow: " + noOfChannels + Config.NEW_LINE;
                                headerInfo += "# Czestotliwosc probkowania: " + frequencyHz + " [Hz]" + Config.NEW_LINE;
                                headerInfo += "# Czestotliwosc probkowania: " + frequency + " [" + Config.SECOND_TYPES[countUnit] + "]" + Config.NEW_LINE;
                                headerInfo += "# Czas pomiaru: " + mMeasureTimeInS + " [" + Config.SECOND_TYPES[5 - cbUnits.SelectedIndex] + "]" + Config.NEW_LINE;
                                headerInfo += "# Liczba probek na kanal: " + totalNoOfSamples + Config.NEW_LINE;
                                headerInfo += "# Laczna liczba probek: " + totalNoOfSamples * noOfChannels + Config.NEW_LINE;

                                string channelInfo = "";
                                for (int i = 0; i < 8; i++)
                                {
                                    if ((combobox_values[i] != (Imports.Range)(-1))) channelInfo += Config.CHANNEL_NAMES[i] + Config.DELIMITER;
                                }
                                headerInfo += "# " + channelInfo + Config.NEW_LINE;

                                Encoding en = Encoding.ASCII;
                                int headerBytesCount = en.GetByteCount(headerInfo);
                                freeBytesForHeader = new byte[Config.BYTES_FOR_HEADER - headerBytesCount];

                               // sw.Write(headerInfo);
                            }
                            //using (FileStream fs = File.Open(filePathCSV, FileMode.Append))
                            //{
                            //    fs.Write(freeBytesForHeader, 0, freeBytesForHeader.Length);
                            //}

                            for (int f = 0; f < _noOfSamples; f = f + 1000000)
                            {
                                FileInfo file = new FileInfo(filePathCSV);
                                if ((file.Length + (8000000 * noOfChannels)) > maxFileSizeMB) // czy rozmiar pliku + max rozmiar nastepnego przebiegu jest mnijszy od zakladanego rozmiaru, 8000000 bo 8 bajty ma jeden pomiar * 1000000 bo tyle skokow jest w petli for
                                {
                                    fileCounter++;
                                    filePathCSV = filePathCSVBegin + fileCounter + ".csv";
                                }
                                using (StreamWriter sw = new StreamWriter(filePathCSV, true))
                                {
                                    for (int i = 0; i < 1000000; i++)
                                    {
                                        for (int ch = 0; ch < 8; ch++)
                                        {
                                            if (combobox_values[ch] != (Imports.Range)(-1))
                                            {
                                                sw.Write(adc_to_mv(maxPinned[ch].Target[f + i], combobox_values[ch]));

                                                if (noOfChannels > 1)
                                                {
                                                    sw.Write(Config.DELIMITER);
                                                }
                                            }
                                        }
                                        sw.Write(Config.NEW_LINE);
                                    }
                                }
                            }
                        }
                    }
                    else
                    {
                        MessageBox.Show("data collection aborted", "Data Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        this.Close();
                    }

                }
                    MessageBox.Show("End");
                 //   Dispose();
            }
        }

        private void StreamCallback(short handle, int noOfSamples, uint startIndex, short overflow, uint triggerAt, short triggered, short autoStop, IntPtr pVoid)
        {
            _ready = true;
            _noOfSamples = noOfSamples;
        }

        private void BlockCallback(short handle, short status, IntPtr pVoid)
        {
            _ready = true;
        }

        private void tbPath_Click(object sender, EventArgs e)
        {
            DialogResult result = folderBrowserDialog1.ShowDialog();
            if (result == DialogResult.OK)
            {
                tbPath.Text = this.folderBrowserDialog1.SelectedPath;
            }
        }

        private int adc_to_mv(int raw, Imports.Range range)
        {
            return (raw * inputRanges[(int)(range)]) / Imports.MaxValue;
        }

        /// <summary>
        /// check user parameters
        /// </summary>
        private Boolean check(int noOfChannels, uint totalNoOfSamples)
        {
            Boolean returnValue = true;

            errorProvider1.Clear();
            errorProvider2.Clear();
            errorProvider3.Clear();

            if (tbMaxFileSizeMB.Text.Length == 0)
            {
                errorProvider1.SetError(tbMaxFileSizeMB, "Field cannot be empty!");
                returnValue = false;
            }
            if (tbFrequencyHz.Text.Length == 0)
            {
                errorProvider2.SetError(tbFrequencyHz, "Field cannot be empty!");
                returnValue = false;
            }
            if (tbMeasureTime.Text.Length == 0)
            {
                errorProvider3.SetError(tbMeasureTime, "Field cannot be empty!");
                returnValue = false;
            }
            if (noOfChannels <= 0)
            {
                MessageBox.Show("Choose at least one channel!");
                returnValue = false;
            }
            //if (((int)totalNoOfSamples * noOfChannels) > 250000000)
            //{
            //    MessageBox.Show("Łączna liczba próbek przekracza rozmiar bufora! Zmiejsz liczbę kanałów, czas próbkowania lub częstotliwość próbkowania.");
            //    returnValue = false;
            //}
            return returnValue;
        }

        private static void AddText(FileStream fs, string value)
        {
            if (value != null && value.Length > 0 && value.Substring(0, 1) == "#")
            {
                byte[] info = new UTF8Encoding(true).GetBytes(value);
                fs.Write(info, 0, info.Length);
            }
        }
    }
}
